import { Injectable } from '@angular/core';
import { HttpResponse } from '@angular/common/http';
import { Resolve, ActivatedRouteSnapshot, Router } from '@angular/router';
import { Observable, of, EMPTY } from 'rxjs';
import { mergeMap } from 'rxjs/operators';

import { IReviewRating, ReviewRating } from '../review-rating.model';
import { ReviewRatingService } from '../service/review-rating.service';

@Injectable({ providedIn: 'root' })
export class ReviewRatingRoutingResolveService implements Resolve<IReviewRating> {
  constructor(protected service: ReviewRatingService, protected router: Router) {}

  resolve(route: ActivatedRouteSnapshot): Observable<IReviewRating> | Observable<never> {
    const id = route.params['id'];
    if (id) {
      return this.service.find(id).pipe(
        mergeMap((reviewRating: HttpResponse<ReviewRating>) => {
          if (reviewRating.body) {
            return of(reviewRating.body);
          } else {
            this.router.navigate(['404']);
            return EMPTY;
          }
        })
      );
    }
    return of(new ReviewRating());
  }
}
