import { TestBed } from '@angular/core/testing';
import { HttpResponse } from '@angular/common/http';
import { HttpClientTestingModule } from '@angular/common/http/testing';
import { ActivatedRouteSnapshot, ActivatedRoute, Router, convertToParamMap } from '@angular/router';
import { RouterTestingModule } from '@angular/router/testing';
import { of } from 'rxjs';

import { IWatchListEntry, WatchListEntry } from '../watch-list-entry.model';
import { WatchListEntryService } from '../service/watch-list-entry.service';

import { WatchListEntryRoutingResolveService } from './watch-list-entry-routing-resolve.service';

describe('WatchListEntry routing resolve service', () => {
  let mockRouter: Router;
  let mockActivatedRouteSnapshot: ActivatedRouteSnapshot;
  let routingResolveService: WatchListEntryRoutingResolveService;
  let service: WatchListEntryService;
  let resultWatchListEntry: IWatchListEntry | undefined;

  beforeEach(() => {
    TestBed.configureTestingModule({
      imports: [HttpClientTestingModule, RouterTestingModule.withRoutes([])],
      providers: [
        {
          provide: ActivatedRoute,
          useValue: {
            snapshot: {
              paramMap: convertToParamMap({}),
            },
          },
        },
      ],
    });
    mockRouter = TestBed.inject(Router);
    jest.spyOn(mockRouter, 'navigate').mockImplementation(() => Promise.resolve(true));
    mockActivatedRouteSnapshot = TestBed.inject(ActivatedRoute).snapshot;
    routingResolveService = TestBed.inject(WatchListEntryRoutingResolveService);
    service = TestBed.inject(WatchListEntryService);
    resultWatchListEntry = undefined;
  });

  describe('resolve', () => {
    it('should return IWatchListEntry returned by find', () => {
      // GIVEN
      service.find = jest.fn(id => of(new HttpResponse({ body: { id } })));
      mockActivatedRouteSnapshot.params = { id: 123 };

      // WHEN
      routingResolveService.resolve(mockActivatedRouteSnapshot).subscribe(result => {
        resultWatchListEntry = result;
      });

      // THEN
      expect(service.find).toBeCalledWith(123);
      expect(resultWatchListEntry).toEqual({ id: 123 });
    });

    it('should return new IWatchListEntry if id is not provided', () => {
      // GIVEN
      service.find = jest.fn();
      mockActivatedRouteSnapshot.params = {};

      // WHEN
      routingResolveService.resolve(mockActivatedRouteSnapshot).subscribe(result => {
        resultWatchListEntry = result;
      });

      // THEN
      expect(service.find).not.toBeCalled();
      expect(resultWatchListEntry).toEqual(new WatchListEntry());
    });

    it('should route to 404 page if data not found in server', () => {
      // GIVEN
      jest.spyOn(service, 'find').mockReturnValue(of(new HttpResponse({ body: null as unknown as WatchListEntry })));
      mockActivatedRouteSnapshot.params = { id: 123 };

      // WHEN
      routingResolveService.resolve(mockActivatedRouteSnapshot).subscribe(result => {
        resultWatchListEntry = result;
      });

      // THEN
      expect(service.find).toBeCalledWith(123);
      expect(resultWatchListEntry).toEqual(undefined);
      expect(mockRouter.navigate).toHaveBeenCalledWith(['404']);
    });
  });
});
