import { Component, Input, OnInit } from '@angular/core';
import { AccountService } from '../../core/auth/account.service';
import { WatchlistManager } from '../watchlist/watchlist-manager';
import { AlertService } from '../../core/util/alert.service';
import { ExtendedSearchResultDTO, SearchResultDTO } from '../model/search/search-result-dto.model';

@Component({
  selector: 'jhi-bookmark-toggle',
  templateUrl: './bookmark-toggle.component.html',
})
export class BookmarkToggleComponent implements OnInit {
  @Input() exercise: ExtendedSearchResultDTO | undefined;

  isBookmarked = false;
  isLoadingBookmark = false;

  constructor(private jhiAlertService: AlertService, private watchlistManager: WatchlistManager, private accountService: AccountService) {}

  ngOnInit(): void {
    this.isBookmarked = this.watchlistManager.isExerciseOnCurrentWatchlist(this.exercise!);
  }

  isOnCurrentWatchlist(e: SearchResultDTO | undefined): boolean {
    return e ? this.watchlistManager.isExerciseOnCurrentWatchlist(e) : false;
  }

  isLoggedIn(): boolean {
    return this.accountService.isAuthenticated();
  }

  handleForCurrentWatchlist(): void {
    if (!this.isLoadingBookmark) {
      this.isLoadingBookmark = true;
      this.watchlistManager.handleCheckForCurrentWatchlist(this.exercise!)?.then(() => {
        this.isBookmarked = !this.isBookmarked;
        this.exercise!.numberOfWatchlistEntries = !this.isBookmarked
          ? this.exercise!.numberOfWatchlistEntries--
          : this.exercise!.numberOfWatchlistEntries++;
        this.isLoadingBookmark = false;
      });
    }
  }
}
